<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class WPAPRegisterFieldsHandler
{
    static function init()
    {
        add_action( 'wp_ajax_nopriv_wpap_register', [ __CLASS__, 'registerValidation' ] );
        add_action( 'wp_ajax_wpap_user_edit', [ __CLASS__, 'userEditValidation' ] );
    }

    static function registerValidation()
    {
        if ( ! isset( $_POST['wpap_register_nonce'] ) or ! wp_verify_nonce( $_POST['wpap_register_nonce'], 'register_nonce' ) ) {
            wp_send_json( [ 'status' => 'error', 'msg' => __( 'Invalid Request.', 'arvand-panel' ) ] );
        }

        $posts = get_posts([
            'post_type' => 'wpap_reg_field',
            'order' => 'ASC',
            'orderby' => 'menu_order',
            'numberposts' => -1,
        ]);

        $messages = [];
        $user = [];

        foreach ( $posts as $post ) {
            $method_name = esc_html( "{$post->post_title}_settings" );
            $settings = WPAPFiledSettings::$method_name( $post->post_content );

            if ( isset( $_POST[ $post->post_title ] ) and in_array( $post->post_title, WPAPField::supported_fields( 'single' ) ) ) {
                $display_name = isset( $settings['placeholder'] ) ? esc_html( $settings['placeholder'] ) : esc_html( $settings['label'] );

                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST[ $post->post_title ] ) ) {
                    $messages[] = sprintf( __( '%s must not be empty.', 'arvand-panel' ), $display_name );
                }

                if ( isset( $settings['min'] ) and strlen( $_POST[ $post->post_title ] ) < $settings['min'] ) {
                    $messages[] = sprintf( __( 'Minimum %s letters must be %d', 'arvand-panel' ), $display_name, $settings['min'] );
                }

                if ( $post->post_title === 'user_login' and username_exists( $_POST['user_login'] ) ) {
                    $messages[] = __( 'Username already exist.', 'arvand-panel' );
                }

                if ( $post->post_title === 'user_email' ) {
                    if ( ! is_email( $_POST['user_email'] ) ) {
                        $messages[] = __( 'Invalid Email', 'arvand-panel' );
                    }

                    if ( email_exists( $_POST['user_email'] ) ) {
                        $messages[] = __( 'Email already exist.', 'arvand-panel' );
                    }
                }

                if ( $post->post_title === 'user_pass' ) {
                    if ( empty( $_POST['user_pass'] ) ) {
                        $messages[] = __( 'Password must not be empty.', 'arvand-panel' );
                    }

                    if ( $_POST['user_pass_confirm'] !== $_POST['user_pass'] ) {
                        $messages[] = __( 'Two passwords do not match.', 'arvand-panel' );
                    }
                }

                if ( $post->post_title === 'mobile' ) {
					$phone = WPAPSMS::checkPhone( $_POST['mobile'] );

					if ( ( $settings['required'] === 'required' or ! empty( $_POST['mobile'] ) ) and ! $phone ) {
                        $messages[] = __( 'Invalid mobile number.', 'arvand-panel' );
                    }

                    if ( WPAPUser::checkPhoneFields( $phone ) ) {
                        $messages[] = __( 'This mobile number already exist.', 'arvand-panel' );
                    }
                }

                if ( in_array( $post->post_title, [ 'user_login', 'user_email', 'user_pass' ] ) ) {
                    $user[ $post->post_title ] = sanitize_text_field( wpap_en_num( $_POST[ $post->post_title ] ) );
                } elseif ( $post->post_title !== 'mobile' ) {
                    $user[ $post->post_title ] = sanitize_text_field( $_POST[ $post->post_title ] );
                }
            }

            $single_input = in_array( $post->post_title, WPAPField::supported_fields( 'single_input' ) );
            $multi_option = in_array( $post->post_title, WPAPField::supported_fields( 'multi_option' ) );

            if ( $single_input and isset( $_POST[ $settings['name'] ] ) ) {
                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST[ $settings['name'] ] ) ) {
                    $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['placeholder'] ) );
                }

                if ( strlen( $_POST[ $settings['name'] ] ) < $settings['min'] ) {
                    $messages[] = sprintf( __( 'Minimum %s letters must be %d', 'arvand-panel' ), esc_html( $settings['placeholder'] ), esc_html( $settings['min'] ) );
                }
            }

            if ( $multi_option and ! isset( $_POST[ $settings['name'] ] ) and $settings['required'] === 'required' ) {
                $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['label'] ) );
            }
        }

        $register = wpap_register_options();

        if ( $register['enable_agree'] and $register['agree_required'] and ! isset( $_POST['agree'] ) ) {
            $messages[] = __( 'You must agree to the terms.', 'arvand-panel' );
        }

        if ( $register['enable_recaptcha'] and ! empty( $register['recaptcha_secret_key'] ) ) {
            $secret = sanitize_text_field( $register['recaptcha_secret_key'] );
            $verify_response = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret=' . $secret . '&response=' . $_POST['g-recaptcha-response'] );
            $response_data = json_decode( $verify_response );

            if ( ! $response_data->success ) {
                $messages[] = __( 'Recaptcha verification failed, please try again.', 'arvand-panel' );
            }
        }

		do_action( 'wpap_register_fields_validate', $messages );

        if ( count( $messages ) ) {
            wp_send_json( [ 'status' => 'error', 'msg' => $messages ] );
        }

        wp_send_json( self::saveRegForm( $user, $_POST, $posts ) );
    }

    static function saveRegForm( $user, $inputs, $posts )
    {
        $register = wpap_register_options();

        if ( ! $register['enable_admin_approval'] and $register['register_activation'] ) {
            $user['user_activation_key'] = sha1( wp_generate_password( 20 ) . $inputs['user_email'] . uniqid( time(), true ) );
        }

        $user_id = wp_insert_user( $user );

        if ( is_wp_error( $user_id ) ) {
            return [ 'status' => 'error', 'msg' => __( 'There is a problem with registration.', 'arvand-panel' ) ];
        }

        $multi = WPAPField::supported_fields( 'multi' );

        foreach ( $posts as $post ) {
            $settings = unserialize( $post->post_content );

            if ( in_array( $post->post_title, $multi ) and isset( $inputs[ $settings['name'] ] ) ) {
                $field = is_array( $inputs[ $settings['name'] ] ) ? maybe_serialize( wpap_sanitize_array_fields( $inputs[ $settings['name'] ] ) ) : sanitize_text_field( $inputs[ $settings['name'] ] );
                add_user_meta( $user_id, sanitize_text_field( $settings['meta_key'] ), $field );
            }

            if ( $post->post_name === 'mobile' and isset( $inputs['mobile'] ) and isset( $settings['required'] ) and $settings['required'] ) {
                update_user_meta( $user_id, 'wpap_user_phone_number', sanitize_text_field( wpap_en_num( $inputs['mobile'] ) ) );
            }
        }

        if ( $register['enable_agree'] and isset( $inputs['agree'] ) ) {
            add_user_meta( $user_id, 'wpap_agree_to_terms', 1 );
        }

        add_user_meta( $user_id, 'wpap_user_status', 0 );
        $user = get_user_by( 'ID', $user_id );

		do_action( 'wpap_register_fields_save', $inputs, $user );

        $email_opt = wpap_email_options();
        self::regMail( get_bloginfo( 'admin_email' ), $user, esc_html( $email_opt['reg_email_subject'] ), $email_opt['reg_email_content'] );

        if ( ! $register['enable_admin_approval'] and $register['register_activation'] ) {
            self::regMail( $user->user_email, $user, $email_opt['activation_email_subject'], $email_opt['activation_email'], true );
            return [ 'status' => 'success', 'msg' => __( 'Registration completed successfully. An account activation link was sent to your email.', 'arvand-panel' ), 'redirect' => false ];
        }

        if ( $register['enable_admin_approval'] ) {
            return [ 'status' => 'success', 'msg' => __( 'Registration completed successfully. Your account awaiting admin approval.', 'arvand-panel' ), 'redirect' => false ];
        }

	    wp_clear_auth_cookie();
	    wp_set_current_user( $user->ID );
	    wp_set_auth_cookie( $user->ID );
        return [ 'status' => 'success', 'msg' => __( 'Registration completed successfully', 'arvand-panel' ), 'redirect' => true ];
    }

    static function regMail( $email, $user, $subject, $content, $activation = false )
    {
        $headers = [ 'Content-Type: text/html; charset=UTF-8' ];
        $search = [ '[first_name]', '[last_name]', '[user_login]', '[user_email]', '[site_name]', '[site_url]', '[activation_link]' ];
        $site_link = '<a href="' . esc_url( site_url() ) . '">' . site_url() . '</a>';
        $replace = [ $user->first_name, $user->last_name, $user->user_login, $user->user_email, get_bloginfo( 'name' ), $site_link ];

        if ( $activation ) {
            $pages = wpap_pages_options();
            $login_url = $pages['login_page_id'] ? get_permalink( $pages['login_page_id'] ) : wp_login_url();
            $activation_link = '<a href="' . esc_url( $login_url . '?email=' . $email . '&key=' . $user->user_activation_key ) . '">' . __( 'activation link', 'arvand-panel' ) . '</a>';
			$replace[] = apply_filters( 'wpap_email_activation_link', $activation_link, $email, $user, $content );
        }

        wp_mail( $email, $subject, wpap_email_template( str_replace( $search, $replace, $content ) ), $headers );
    }

    static public function userEditValidation()
    {
	    $current_user = wp_get_current_user();

	    if ( $current_user->user_login === WPAP_DEMO ) {
		    wp_send_json( [ 'status' => 'error', 'msg' => __( 'کاربر دمو قادر به ذخیره سازی نیست.', 'arvand-panel' ) ] );
	    }

        if ( ! isset( $_POST['user_edit_nonce'] ) or ! wp_verify_nonce( $_POST['user_edit_nonce'], 'user_edit_nonce' ) ) {
	        wp_send_json( [ 'status' => 'error', 'msg' => __( 'Invalid Request.', 'arvand-panel' ) ] );
        }

        $messages = [];

        if ( isset( $_FILES['profile_pic'] ) and ! empty( $_FILES['profile_pic']['name'] ) ) {
            $response = WPAPUser::upload_pro_pic( $_FILES['profile_pic'] );

            if ( isset( $response['error'] ) ) {
	            $messages[] = $response['error'];
            }
        }

        $supported_fields = WPAPField::supported_fields();
        $user_data = [];
        $register = wpap_register_options();
        $multi = WPAPField::supported_fields( 'multi' );

	    $posts = get_posts([
		    'post_type' => 'wpap_reg_field',
		    'order' => 'ASC',
		    'orderby' => 'menu_order',
		    'numberposts' => -1,
	    ]);

        foreach ( $posts as $post ) {
            $settings = unserialize( $post->post_content );

            if ( in_array( $post->post_title, $supported_fields ) and $settings['user_edit_display'] === 'hide' ) {
                continue;
            }

            if ( $post->post_title === 'first_name' and isset( $_POST['first_name'] ) ) {
                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST['first_name'] ) ) {
	                $messages[] = __( 'Firstname must not be empty.', 'arvand-panel' );
                }

                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and strlen( $_POST['first_name'] ) < $settings['min'] ) {
	                $messages[] = sprintf( __( 'Minimum firstname letters must be %d', 'arvand-panel' ), $settings['min'] );
                }

                $user_data['first_name'] = sanitize_text_field( $_POST['first_name'] );
            }

            if ( $post->post_title === 'last_name' and isset( $_POST['last_name'] ) ) {
                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST['last_name'] ) ) {
	                $messages[] = __( 'Lastname must not be empty.', 'arvand-panel' );
                }

                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and strlen( $_POST['last_name'] ) < $settings['min'] ) {
	                $messages[] = sprintf( __( 'Minimum lastname letters must be %d', 'arvand-panel' ), $settings['min'] );
                }

                $user_data['last_name'] = sanitize_text_field( $_POST['last_name'] );
            }

            if ( $register['register_activation'] and empty( $current_user->user_email ) and $post->post_title === 'user_email' and isset( $_POST['user_email'] ) ) {
                if ( empty( $_POST['user_email'] ) ) {
	                $messages[] = __( 'Email must not be empty.', 'arvand-panel');
                }

                if ( ! is_email( $_POST['user_email'] ) ) {
	                $messages[] = __( 'Invalid email.', 'arvand-panel');
                }

                if ( email_exists( $_POST['user_email'] ) ) {
	                $messages[] = __( 'Email already exists.', 'arvand-panel');
                }

                $user_data['user_email'] = sanitize_email( $_POST['user_email'] );
            }

            if ( $post->post_title === 'display_name' and isset( $_POST['display_name'] ) ) {
                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST['display_name'] ) ) {
	                $messages[] = __( 'Display name must not be empty.', 'arvand-panel' );
                }

                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and strlen( $_POST['display_name'] ) < $settings['min'] ) {
	                $messages[] = sprintf( __( 'Minimum display name letters must be %d', 'arvand-panel' ), $settings['min'] );
                }

                $user_data['display_name'] = sanitize_text_field( $_POST['display_name'] );
            }

            if ( $post->post_title === 'user_url' and isset( $_POST['user_url'] ) ) {
                if ( isset( $settings['required'] ) and $settings['required'] === 'required' and empty( $_POST['user_url'] ) ) {
	                $messages[] = __( 'Website must not be empty.', 'arvand-panel' );
                }

                $user_data['user_url'] = sanitize_text_field( $_POST['user_url'] );
            }

            if ( $post->post_title === 'text_field' ) {
                if ( isset( $_POST[ $settings['name'] ] ) ) {
                    if ( $settings['required'] === 'required' and empty( $_POST[ $settings['name'] ] ) ) {
	                    $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['placeholder'] ) );
                    }

                    if ( strlen( $_POST[ $settings['name'] ] ) < $settings['min'] ) {
	                    $messages[] = sprintf( __('Minimum %s letters must be %d', 'arvand-panel' ), esc_html( $settings['placeholder'] ), esc_html( $settings['min'] ) );
                    }
                }
            }

            if ( $post->post_title === 'radio' ) {
                if ( ! isset( $_POST[ $settings['name'] ] ) and $settings['required'] === 'required' ) {
	                $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['label'] ) );
                }
            }

            if ( $post->post_title === 'checkbox' ) {
                if ( ! isset( $_POST[ $settings['name'] ] ) and $settings['required'] === 'required' ) {
	                $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['label'] ) );
                }
            }

            if ( $post->post_title === 'drop_down' ) {
                if ( ( ! isset( $_POST[ $settings['name'] ] ) or ! in_array( $_POST[ $settings['name'] ], $settings['options'] ) ) and $settings['required'] === 'required' ) {
	                $messages[] = sprintf( __( '%s is required.', 'arvand-panel' ), esc_html( $settings['label'] ) );
                }
            }

            if ( $post->post_title === 'number_field' ) {
                if ( isset( $_POST[ $settings['name'] ] ) ) {
                    if ( $settings['required'] === 'required' and empty( $_POST[ $settings['name'] ] ) ) {
	                    $messages[] = sprintf( __('%s is required.', 'arvand-panel'), esc_html( $settings['placeholder'] ) );
                    }

                    if ( strlen( $_POST[ $settings['name'] ] ) < $settings['min'] ) {
	                    $messages[] = sprintf( __('Minimum %s letters must be %d', 'arvand-panel' ), esc_html( $settings['placeholder'] ), esc_html( $settings['min'] ) );
                    }
                }
            }
        }

		do_action( 'wpap_user_edit_validate_fields', $current_user, $messages );

        if ( count( $messages ) > 0 ) {
	        wp_send_json( [ 'status' => 'error', 'msg' => $messages ] );
        }

        $user_data['ID'] = $current_user->ID;
        $user_update = wp_update_user( $user_data );

        if ( is_wp_error( $user_update ) ) {
	        wp_send_json( [ 'status' => 'error', 'msg' => __( 'There is a problem editing the profile.', 'arvand-panel' ) ] );
        }

        foreach ( $posts as $post ) {
            if ( in_array( $post->post_title, $multi ) ) {
                $field = unserialize( $post->post_content );

                if ( ! isset( $_POST[ $field['name'] ] ) ) {
                    delete_user_meta( $current_user->ID, $field['meta_key'] );
                }

                if ( isset( $_POST[ $field['name'] ] ) ) {
                    if ( is_array( $_POST[ $field['name'] ] ) ) {
                        update_user_meta( $current_user->ID, sanitize_text_field( $field['meta_key'] ), maybe_serialize( wpap_sanitize_array_fields( $_POST[ $field['name'] ] ) ) );
                    } else {
                        update_user_meta( $current_user->ID, sanitize_text_field( $field['meta_key'] ), sanitize_text_field( $_POST[ $field['name'] ] ) );
                    }
                }
            }
        }

	    do_action( 'wpap_user_edit_fields_save', $current_user );

	    wp_send_json( [ 'status' => 'success', 'msg' => __( 'Changes applied successfully.', 'arvand-panel' ) ] );
    }
}

WPAPRegisterFieldsHandler::init();